% -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-
%               SAMPLE CODE FOR THE PERFORMANCE PREDICTION CHALLENGE
%             ADAPTED FOR THE FEATURE EXTRACTION COURSE WS 2005/06
%             Isabelle Guyon -- isabelle@clopinet.com -- October 2005
% -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-

% DISCLAIMER: ALL INFORMATION, SOFTWARE, DOCUMENTATION, AND DATA ARE PROVIDED "AS-IS" 
% ISABELLE GUYON AND/OR OTHER ORGANIZERS DISCLAIM ANY EXPRESSED OR IMPLIED WARRANTIES, 
% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
% FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY OF NON-INFRIGEMENT OF ANY THIRD PARTY'S 
% INTELLECTUAL PROPERTY RIGHTS. IN NO EVENT SHALL ISABELLE GUYON AND/OR OTHER ORGANIZERS 
% BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
% ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF SOFTWARE, DOCUMENTS, 
% MATERIALS, PUBLICATIONS, OR INFORMATION MADE AVAILABLE FOR THE CHALLENGE. 
%
% ---------
% Tested and modified by: 
% Gideon Dror, gideon@mta.ac.il
% Amir Reza Saffari Azar, amir@ymer.org.
% September 2005
% ---------

clear all

% -o-|-o-|-o-|-o-|-o-|-o-|-o- BEGIN USER-PREFERENCES -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-

% 1) User-defined directories (no slash at the end of the names):
% --------------------------------------------------------------
% The present set up supposes that you are now in the directory sample_code
% and you have the following directory tree, where Challenge is you project directory:
% Exercises/Data
% Exercises/Results
% Exercises/Zipped
% Exercises/Models
% Exercises/Clop

% Put in the directory my_root the sample code for the class, including
% this file.

my_root     = pwd;                  % Change that to the directory of your project

data_dir    = [my_root '/Data'];    %'C:\Users\Isabelle\Projects\ETH\Data';    
                                    % Path to the five data directories downloaded
                                    % from http://clopinet.com/isabelle/Projects/NIPS2003/
                                    % or http://www.nipsfsc.ecs.soton.ac.uk/datasets/
                                    % (ARCENE, DEXTER, DOROTHEA, GISETTE, and MADELON ).
resu_dir    = [my_root '/Results']; % Where the results will end up.    
zip_dir     = [my_root '/Zipped'];  % Zipped files with results ready to go!
model_dir   = [my_root '/Models'];  % Where the trained models will end up.
                                    % Note: you need not include the models in
                                    % your submissions, unless you want to
                                    % qulify for a bonus entry. In this
                                    % example, the models are save in full
                                    % in the model_dir directory and a lean
                                    % version (with the hyperparameters and
                                    % the smallest trained entities) is
                                    % save in the resu_dir directory.
                                    
ForceOverWrite = 1;                 % Change this value to 0 if you want to be warned when
                                    % a file already exists before saving a result or model.

% If you just downloaded the sample code, you do not have the spider
% and Challenge Learning Object Package (CLOP). 
% You may download them from: 
% http://clopinet.com/isabelle/Projects/modelselect/Clop.zip. 
% But you do not need to, you can point the code_dir to the sample_code
% directory:
code_dir    = [my_root '/Clop'];    %'C:\Users\Isabelle\Projects\NSF\Clop';    
                                    % Path to the sample code or the 
                                    % Challenge Learning Objects Package (CLOP).

% 2) Choose your data and models
% ------------------------------
%dataset     = {'arcene', 'dexter', 'dorothea', 'gisette', 'madelon'};
%modelset    = {};
dataset     = {'arcene', 'dexter', 'dorothea', 'gisette', 'madelon'};
modelset    = {'Prepro+linearSVC'}; % This should be an array of model example names
                                    % e.g. {'zarbi', 'Prepro+linearSVC'}
                                    % If you leave this array is empty, it will 
                                    % be replaced by a default list of model names.                              
                                    % For a list of model examples, use:
                                    % > model_examples

% -o-|-o-|-o-|-o-|-o-|-o-|-o- END USER-PREFERENCES -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-
fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-\n');

% Set the path and defaults properly; create directories
% ------------------------------------------------------
addpath(code_dir);
if exist('use_spider_clop.m') == 2, 
    use_spider_clop(code_dir);
elseif exist('code_version.m') == 2, 
    fprintf('Sample code version : %s\n', code_version('sample_code'));
else disp 'ERROR: Wrong code path. Check your directories and path variables.';
    if exist('README.txt') == 2, type README.txt; end
    if exist('Data/README.txt') == 2, type Data/README.txt; end
    if exist('Clop/README.txt') == 2, type Clop/README.txt; end
    return; 
end
if isempty(modelset), modelset =  model_examples; end
makedir(resu_dir);
makedir(zip_dir);
makedir(model_dir);

% LOOP OVER DATASETS 
% ==================
for k = 1:length(dataset)
    
	data_name   = dataset{k};
    
    fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-      %s      -o-|-o-|-o-|-o-|-o-|-o-\n', upper(data_name));
    fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-\n\n');
    
    % Create a data structure and check the data
    %===========================================
    fprintf('-- %s loading data --\n', upper(data_name));
    input_dir   = [data_dir '/' upper(data_name)];
    input_name  = [input_dir '/' data_name];
    
    D   = create_data_struct(input_name);
    % Each member of D (D.train, D.valid, and D.test) 
    % is a data object (a structure) with members X and Y (if Y is given). 
    
    fprintf('-- %s data loaded --\n', upper(data_name));
    % Note: the data are saved as a Matlab structure 
    % so they will load faster the second time around. 

    % LOOP OVER MODELS 
    % ================
    for j = 1:length(modelset)
        
        model_name  = modelset{j};
        
        % Build a model
        %==============
        fprintf('-- %s-%s building  model\n', upper(data_name), upper(model_name));
        my_model    = model_examples(model_name, data_name);
        % Note: "model_examples" calls a model constructor and returns a learning object.
        % Enter at the prompt "> type model_examples" to view the examples.
        % All learning objects have the two methods "train" and "test".
        % To see the data members, type at the prompt "> struct(my_model)".
        fprintf('-- %s-%s model built\n', upper(data_name), upper(model_name));
        
        % Train the model
        %================
        tic;
        fprintf('-- %s-%s training model\n', upper(data_name), upper(model_name));
        % Call the method "train" of the object "my_model":
        [train_output my_model] = train(my_model, D.train);
        
        fprintf('-- %s-%s model trained in %5.2f seconds\n', upper(data_name), upper(model_name), toc);
        % Note: The "train" method returns the predicted discriminant values
        % in train_output.X; the target values are in train_output.Y.
        guessed_ber = balanced_errate(train_output.X,train_output.Y); % Rough guess: the training error!
        fprintf('Making the very tentative BER guess of %5.2f%%\n', 100*guessed_ber);
        
        % Note: This example does not perform any hyperparameter or model
        % selection, which is one objective of the competition. It shows
        % only how to train/test/save. One should have some kind of
        % cross-validation or some method of evaluating guessed_ber in a
        % more meaningful way to perform decently in the challenge.
        
        % Test the model
        %===============
        tic;
        fprintf('-- %s-%s testing model\n', upper(data_name), upper(model_name));
        set_name    = fieldnames(D); % 'train', 'valid' and 'test'
        
        for i = 1:length(set_name) % Loop over training, validation, and test set
            % Call the method "test" of the object "my_model" 
            % on D.train, D.valid, or D.test:
            output  = test(my_model, D.(set_name{i})); 
            discriminant{i} = output.X; % The "test" method returns the predicted discriminant values in output.X
            target = output.Y; % If the targets were provided in the dataset, they are copied in output.Y
            if ~isempty(target)
                ber = balanced_errate(discriminant{i},target);
                guess_error = challenge_error(guessed_ber, ber);
                ebar        = error_bar(ber, length(find(target>0)));
                fprintf('%s, BER=%5.2f +-%5.2f%%, guess_error=%5.2f%%\n', ...
                    set_name{i}, 100*ber, 100*ebar, 100*guess_error); 
            end
        end
        
        fprintf('-- %s-%s model tested in %5.2f seconds\n', upper(data_name), upper(model_name), toc);  
        
        % Save the results
        %=================
        fprintf('-- %s-%s saving the results\n', upper(data_name), upper(model_name));
        resu_name   = [resu_dir '/' model_name ];
        makedir(resu_name);
        resu_name   = [resu_name '/' data_name]; 
        
        for i=1:length(set_name) % Loop over 'train', 'valid', and 'test'
            save_outputs([[resu_name '_' set_name{i}] '.resu'], sign(discriminant{i}), ForceOverWrite);        
            save_outputs([[resu_name '_' set_name{i}] '.conf'], abs(discriminant{i}), ForceOverWrite);        
        end
        
        save_outputs([resu_name '.guess'], guessed_ber, ForceOverWrite);
        fprintf('-- %s-%s results saved as %s*\n', upper(data_name), upper(model_name), resu_name);
        
        % Save the models
        %================
        fprintf('-- %s-%s saving the models\n', upper(data_name), upper(model_name));
        % Remove the biggest structures and arrays to make the model smaller
        % Save the lean model in the results directory to qualify for a "bonus" entry:
        my_lean_model=clean(my_model); 
        mresu_name  = [resu_dir '/' model_name '/' data_name '_model'];
        save_model(mresu_name, my_lean_model, ForceOverWrite);
        % Save also the whole trained model for further reference:
        mresu_name  = [model_dir '/' model_name ];
        makedir(mresu_name);
        mresu_name  = [mresu_name '/' data_name '_model'];
        save_model(mresu_name, my_model, ForceOverWrite);
        fprintf('-- %s-%s models saved as %s*\n', upper(data_name), upper(model_name), mresu_name);
        fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-\n');
    
    end % Loop over models
end % Loop over datasets

% Zip the archives so they are ready to go!
% -----------------------------------------
if ~usejava('jvm'), warning('Java is not loaded, failed to generate ZIP files !!!'); return; end
for k = 1:length(modelset)
    model_name  = modelset{k};
    zip_name    = zipall(model_name, resu_dir, zip_dir);
    if ~isempty(zip_name)
        fprintf('-- %s zip archive created, see %s --\n', upper(model_name), zip_name);
    end
end

    